#define save_flags(x)	x=__save_flags()

extern void disable_irq();
extern void enable_irq();
extern void do_IRQ();
extern unsigned long __save_flags();
extern void restore_flags();
extern int request_irq();
extern void free_irq();
extern void cli();
extern void sti();
