export	_main
_main:
push	bp
mov	bp,sp
push	di
push	si
add	sp,*-4
call	_setup_arch
call	_build_task
mov	-8[bp],ax
mov	ax,-8[bp]
add	ax,*$40
mov	-8[bp],ax
_main.tnum	set	0
_main.a	set	2
	mov ah, 0xe
	mov al, tnum
	int 0x10
call	_taska
_main.tnum	set	0
_main.a	set	2
die:	jmp die
add	sp,*4
pop	si
pop	di
pop	bp
ret
export	_taska
_taska:
a:	mov ax, #0xe41
	int 0x10 
	jmp b
ret
export	_taskb
_taskb:
b:	mov ax, #0xe42
	int 0x10
	jmp a
ret
.data
.bss
.comm	_task,$A0

! 0 errors detected
